﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.IO;
using System.Reflection;
using System.Web;
using Wisej.Web;
using YobsManager.Dal.Mongo;
using YobsManager.Dal.Poco;
using YobsManager.Kernel;
using YobsManager.Screens;
using YobsManager.Screens.Email;
using YobsManager.Screens.Employer;
using YobsManager.Screens.Management;

namespace YobsManager
{
    public partial class YobsDesktop : Desktop
    {
        [ImportMany(typeof(IYobsManagerModule))]
        private List<IYobsManagerModule> Modules { get; set; }
        private User mUser = null;
        public event LogOffDelegate LogOff;
        private MenuData mData = new MenuData(HttpContext.Current);
        public YobsDesktop()
        {

            Initialize();
            lblVersion.Text = string.Format(lblVersion.Text, Application.ProductVersion);
        
        }
        private void Initialize()
        {
            InitializeComponent();
           
            this.lblVersion.Anchor = ((Wisej.Web.AnchorStyles)((Wisej.Web.AnchorStyles.Bottom | Wisej.Web.AnchorStyles.Right)));
            this.pictLogo.Anchor = ((Wisej.Web.AnchorStyles)((Wisej.Web.AnchorStyles.Bottom | Wisej.Web.AnchorStyles.Right)));
            this.modules1.Anchor = ((Wisej.Web.AnchorStyles)((Wisej.Web.AnchorStyles.Bottom | Wisej.Web.AnchorStyles.Left)));
            //  this.modules1.Collapsed(true);
            if (Application.Session["User"] != null)
            {
                mUser = Application.Session["User"] as User;
                lblName.Text = $"{mUser.FirstName} {mUser.Surname}";
              //  AlertBox.Show($"Welcome back {mUser.FirstName}!", MessageBoxIcon.Information);
            }
            LoadMenuData();

            foreach (Role mRole in mUser.Roles)
            {
                if (mRole.Level == 999)
                {
                    mnuManager.Visible = true;
                    break;
                }
            }

            LoadMef();

        }
        private CompositionContainer _container = null;
        private void LoadMef()
        {
            var catalog = new AggregateCatalog();
            catalog.Catalogs.Add(new AssemblyCatalog(Assembly.GetExecutingAssembly()));
            catalog.Catalogs.Add(new DirectoryCatalog(Path.Combine(Application.StartupPath, "Modules")));
            _container = new CompositionContainer(catalog);

            try
            {
                _container.ComposeParts(this);
            }
            catch (CompositionException exc)
            {

                MessageBox.Show(exc.Message);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message);
            }
        }
        private void mnuMenu_Click(object sender, EventArgs e)
        {
            frmMenu mMenu = new frmMenu();
            mMenu.Init();
            mMenu.SendNotification += SendNotification;
            mMenu.FormClosed += MMenu_FormClosed;
            mMenu.Show();
        }

        private void SendNotification(string Message, MessageBoxIcon alertType)
        {
            AlertBox.Show(Message, alertType);
        }

        private void MMenu_FormClosed(object sender, FormClosedEventArgs e)
        {
            LoadMenuData();
        }

        private void LoadMenuData()
        {
            // MainMenuBar = new MenuBar();
            // MainMenuBar.MenuItems.Add(mnuMain);
            List<Dal.Poco.Menu> MenuItems = mData.GetMenuItems();
            List<MenuItem> mRemove = new List<MenuItem>();
            for (int x = 0; x < MainMenuBar.MenuItems.Count; x++)
            {
                if (MainMenuBar.MenuItems[x] != mnuMain && MainMenuBar.MenuItems[x] != mnuManager)
                {
                    mRemove.Add(MainMenuBar.MenuItems[x]);
                }

            }
            foreach (MenuItem mI in mRemove)
            {
                MainMenuBar.MenuItems.Remove(mI);
            }
            int imaxRoleValue = 0;
            foreach (Role mRole in mUser.Roles)
            {
                if (mRole.Level > imaxRoleValue)
                { imaxRoleValue = mRole.Level; }
            }
            foreach (Dal.Poco.Menu mMainMenu in MenuItems)
            {
                if (mMainMenu.Security <= imaxRoleValue)
                {


                    bool bMainMenuFound = false;

                    MenuItem mMainMenuItem = new MenuItem();
                    mMainMenuItem.Text = mMainMenu.Name;
                    mMainMenuItem.Tag = mMainMenu;
                    if (mMainMenu.SubMenu != null)
                    {
                        foreach (Dal.Poco.Menu mChildItem in mMainMenu.SubMenu)
                        {
                            if (mChildItem.Security <= imaxRoleValue)
                            {
                                MenuItem mChildMenuItem = new MenuItem();
                                mChildMenuItem.Click += DynamicMenuClick;
                                mChildMenuItem.Text = mChildItem.Name;
                                mChildMenuItem.Tag = mChildItem;
                                mMainMenuItem.MenuItems.Add(mChildMenuItem);
                            }
                        }
                    }
                    if (mMainMenuItem.MenuItems.Count > 0)
                    {
                        MainMenuBar.MenuItems.Add(MainMenuBar.MenuItems.Count - 1, mMainMenuItem);
                    }
                }

            }

        }

        private void DynamicMenuClick(object sender, EventArgs e)
        {
            MenuItem mI = sender as MenuItem;
            Dal.Poco.Menu mMenu = mI.Tag as Dal.Poco.Menu;
            switch (mMenu.Control.ToLower())
            {
                case "searchcontact":
                    frmSearchContact mSearchContact = new frmSearchContact();
                    mSearchContact.SendNotification += SendNotification;
                    mSearchContact.Init();
                    mSearchContact.Show();
                    break;
                case "searchemployer":
                    frmSearchEmployer mSearchEmployer = new frmSearchEmployer();
                    mSearchEmployer.SendNotification += SendNotification;
                    mSearchEmployer.Show();
                    break;
                case "sendgridmailer":
                    frmSendGridMailer mSendgridMailer = new frmSendGridMailer();
                    mSendgridMailer.SendNotification += SendNotification;
                    mSendgridMailer.Show();
                    break;
                default:
                    foreach(IYobsManagerModule mModule in Modules)
                    {
                        if(mModule.GetType().Name.ToLower()==mMenu.Control.ToLower())
                        {
                            Desktop mForm = mModule as Desktop;
                            
                            Application.LoadTheme("Blue-1");
                            Application.Desktop = mForm;
                            string strdata = mForm.Wallpaper;
                            Application.Desktop.AutoHideTaskbar = true;
                        }
                    }
                    break;

            }
        }

        private void mnuRole_Click(object sender, EventArgs e)
        {
            frmRole mRole = new frmRole();
            mRole.Show();
        }

        private void mnuUser_Click(object sender, EventArgs e)
        {
            frmUser mUser = new frmUser();
            mUser.Show();
        }

        private void mnuLogoff_Click(object sender, EventArgs e)
        {
            Application.Session["User"] = null;
            if (LogOff != null)
            {
                LogOff();

            }
        }

        private void YobsDesktop_Load(object sender, EventArgs e)
        {
            modules1.Collapsed(true);
           
        }
    }
}
